require 'autocommit/detector'
require 'thread'

Thread.abort_on_exception = true

module AutoCommit
  class Gatherer
    Delay = 0.25  # seconds

    def initialize dir, committer
      @dir = dir
      @committer = committer
      @mutex = Mutex.new
      @list = []
      Detector.new dir do |event|
        name = event.absolute_name
        puts "#{name}  (#{event.flags})"  unless /\.git/ =~ name
        trigger_commit event.absolute_name
      end
    end

    def trigger_commit full_name
      in_git_repo = full_name.start_with? File.join(@dir, ".git")
      return  if in_git_repo
      @mutex.synchronize {
        start_thread  if @list.empty?
        @list << Time.now
      }
    end

    def start_thread
      Thread.new {
        delayed_commit
      }
    end

    def delayed_commit
      sleep Delay
      commit
    end

    def commit
      @mutex.synchronize {
        @list.clear
        # Prevent the next commit (>=0.25 seconds later, but still)
        # from interfering with this one:
        # commit from within the synchronized block
        @committer.commit
      }
    end
  end
end
